#ifndef _AUDIOSOURCEQT_H_
#define _AUDIOSOURCEQT_H_


class AudioSourceQT : public vdxunknown<IVDXStreamSource>, public IVDXStreamSourceV3, public IVDXAudioSource {

public:
	AudioSourceQT(InputFileQT *const pp, QTTrack *const qtt);
	~AudioSourceQT();

	// IVDXUnknown methods
	int         VDXAPIENTRY AddRef();
	int         VDXAPIENTRY Release();
	void       *VDXAPIENTRY AsInterface(uint32 iid);

	// IVDXStreamSource methods
	void        VDXAPIENTRY GetStreamSourceInfo(VDXStreamSourceInfo&);
	bool        VDXAPIENTRY Read(sint64 lStart, uint32 lCount, void *lpBuffer, uint32 cbBuffer, uint32 *lBytesRead, uint32 *lSamplesRead);

	const void *VDXAPIENTRY GetDirectFormat();
	int         VDXAPIENTRY GetDirectFormatLen();

	ErrorMode   VDXAPIENTRY GetDecodeErrorMode();
	void        VDXAPIENTRY SetDecodeErrorMode(ErrorMode mode);
	bool        VDXAPIENTRY IsDecodeErrorModeSupported(ErrorMode mode);

	bool        VDXAPIENTRY IsVBR();
	sint64      VDXAPIENTRY TimeToPositionVBR(sint64 us);
	sint64      VDXAPIENTRY PositionToTimeVBR(sint64 samples);

	// IVDXStreamSourceV3 methods
	void        VDXAPIENTRY GetStreamSourceInfoV3(VDXStreamSourceInfoV3& info);

	// IVDXAudioSource methods
	void        VDXAPIENTRY GetAudioSourceInfo(VDXAudioSourceInfo& info);

protected:
	InputFileQT *const parentPtr;
	QTTrack     *const audio;

	IVDXStreamSource::ErrorMode errmode;
};


class AudioSource_PCM : public AudioSourceQT {

public:
	AudioSource_PCM(InputFileQT *const pp, QTTrack *const qtt) : AudioSourceQT(pp, qtt) {}

	bool VDXAPIENTRY Read(sint64 lStart, uint32 lCount, void *lpBuffer, uint32 cbBuffer, uint32 *lBytesRead, uint32 *lSamplesRead);

private:
	static void PCM_word_swap (void *buffer, int bytes);
	static void PCM_fword_swap(void *buffer, int bytes);
	static void PCM_dword_swap(void *buffer, int bytes);
	static void PCM_qword_swap(void *buffer, int bytes);
};


class AudioSource_ima4 : public AudioSourceQT {

public:
	AudioSource_ima4(InputFileQT *const pp, QTTrack *const qtt);
	~AudioSource_ima4();

	bool VDXAPIENTRY Read(sint64 lStart, uint32 lCount, void *lpBuffer, uint32 cbBuffer, uint32 *lBytesRead, uint32 *lSamplesRead);

private:
	const unsigned int inp_blksize;
	const unsigned int out_blksize;

	unsigned char* adpcm_buf;
	unsigned char* adpcm_out;

	void adpcm_decode();
};


#endif  // _AUDIOSOURCEQT_H_

