///////////////////////////////////////////////////////////////////////
// QuickTime plugin for VirtualDub 1.8.8+
// Copyright (C) 2007-2012 fccHandler
// Copyright (C) 1998-2012 Avery Lee
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
///////////////////////////////////////////////////////////////////////

// Error log


#if defined(_MSC_VER) && (_MSC_VER >= 1400)
	#define _CRT_SECURE_NO_WARNINGS
#endif

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#include "ErrorLog.h"


ErrorLog::ErrorLog()
	: text      ((char *)&capacity)
	, text_len  (0)
	, capacity  (0)
{
}


ErrorLog::~ErrorLog()
{
	if (text != (char *)&capacity) delete[] text;
}


void ErrorLog::_append(const char *src, int len)
{
	// Append a string to the log
	// - src must not be NULL
	// - src need not be null-terminated
	// - len must be greater than zero

	if (len > (capacity - text_len))
	{
		const int needed = (((text_len + len) / BLOCK_SIZE) + 1) * BLOCK_SIZE;

		char *tmp = new char[needed];
		if (tmp == NULL) return;

		if (text != (char *)&capacity)
		{
			memcpy(tmp, text, text_len);
			delete[] text;
		}

		capacity = needed - 1;
		text = tmp;
	}

	memcpy(text + text_len, src, len);
	text_len += len;

	text[text_len] = '\0';
}


void ErrorLog::Add(const char *msg, ...)
{
	// Add a formatted message to the log

	if (msg != NULL)
	{
		va_list vl;

		char*   out;	// output buffer for _vsnprintf
		char*   tmp;	// temporary buffer for resizing
		int     max;	// capacity of output buffer - 1
		int     len;	// total number of chars written

		out = new char[BLOCK_SIZE];
		if (out == NULL) return;

		len = 0;
		max = BLOCK_SIZE - 1;

		va_start(vl, msg);

		for (;;)
		{
			// _vsnprintf and _vsnwprintf return the number of characters
			// written, not including the terminating null character, or
			// a negative value if an output error occurs.

			// Beware! This function does not necessarily null-terminate.
			// Therefore send one less than the true buffer size so that
			// we can guarantee room for our required null.

			len = _vsnprintf(out, max, msg, vl);
			if (len >= 0) break;

			// For _vsnprintf, if the number of bytes to write exceeds
			// buffer, then count bytes are written and -1 is returned.

			len = max;  // this much was written so far

			// Increase the output buffer capacity

			tmp = new char[(max + 1) + BLOCK_SIZE];
			if (tmp == NULL) break;

			delete[] out;
			out = tmp;
			max += BLOCK_SIZE;
		}

		if (out != NULL && len > 0)
		{		
			_append(out, len);
			delete[] out;
		}
	}
}
