///////////////////////////////////////////////////////////////////////
// QuickTime plugin for VirtualDub 1.8.8+
// Copyright (C) 2007-2012 fccHandler
// Copyright (C) 1998-2012 Avery Lee
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
///////////////////////////////////////////////////////////////////////

// File Information dialog (part of InputFileQT class)


#if defined(_MSC_VER) && (_MSC_VER >= 1400)
	#define _CRT_SECURE_NO_WARNINGS
#endif

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <commctrl.h>
#include <vfw.h>
#include <stdio.h>

#include "vd2\plugin\vdinputdriver.h"
#include "Unknown.h"

#include "QTTypes.h"
#include "QTTrack.h"

#include "WinFile.h"
#include "ErrorLog.h"
#include "InputOptionsQT.h"
#include "InputFileQT.h"
#include "Resource.h"

extern HMODULE g_hModule;


INT_PTR InputFileQT::LogDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	static HBRUSH hBrush = NULL;

	switch (uMsg) {

    case WM_INITDIALOG:
	{
		InputFileQT *thisPtr = (InputFileQT *)lParam;

		InputOptionsQT::InitDialogTitle(hDlg, " - Import Log");
		SetDlgItemTextA(hDlg, IDC_IMPORT_LOG, thisPtr->log.GetText());

		hBrush = CreateSolidBrush(GetSysColor(COLOR_WINDOW));
		return TRUE;
	}

	case WM_CTLCOLORSTATIC:
		SetBkColor((HDC)wParam, GetSysColor(COLOR_WINDOW));
		return (INT_PTR)hBrush;

	case WM_COMMAND:
		// Note: "IDCANCEL" is the [X] button
		// Return TRUE if we process the message
		if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL)
		{
			EndDialog(hDlg, TRUE);
			return TRUE;
		}
		break;

	case WM_DESTROY:
		DeleteObject((HGDIOBJ)hBrush);
		hBrush = NULL;
		break;
    }

    return FALSE;
}


char *InputFileQT::time_to_string(double seconds)
{
	// There are 86400 seconds in a day,
	// about 32 million seconds in a year.
	// Since we are unlikely to encounter videos
	// of such an extreme duration, we can easily
	// get away with using longs for this math...

	const int buf_size = 32;
	char *ret = new char[buf_size];

	if (ret != NULL)
	{
		int hours, mins, mils;
		int secs = (int)seconds;

		hours = secs / 3600;
		secs  = secs % 3600;
		mins  = secs / 60;
		secs  = secs % 60;
		mils  = (int)(seconds * 1000.0 + 0.5);
		mils  = mils % 1000;

		if (hours > 0)
		{
			_snprintf(ret, buf_size, "%i:%02i:%02i.%03i", hours, mins, secs, mils);
		}
		else
		{
			_snprintf(ret, buf_size, "%i:%02i.%03i", mins, secs, mils);
		}

		ret[buf_size - 1] = '\0';
	}

	return ret;
}


typedef struct {
	__int64       i64FrameTotalSize[2];
	unsigned long dwFrameCount[2];
	unsigned long dwFrameMinSize[2];
	unsigned long dwFrameAvgSize[2];
	unsigned long dwFrameMaxSize[2];
} DlgStreamInfo;


void InputFileQT::UpdateDlgAudio(HWND hDlg, int index)
{
	char buf[64];
	int i, n;

	DlgStreamInfo pInfo = { 0 };

	const WAVEFORMATEX  *wfex;
	const QTTrack       *qtt;

	InputFileQT *thisPtr =
		(InputFileQT *)GetWindowLongPtrA(hDlg, DWLP_USER);

/**************************************************************************
 *      Stream x of x
 **************************************************************************/
	
	for (i = 0, n = 0; i < thisPtr->num_tracks; ++i)
	{
		if (thisPtr->Track[i]->fccType == streamtypeAUDIO) ++n;
	}

	_snprintf(buf, sizeof(buf), "Stream %i of %i", index + 1, n);
	buf[sizeof(buf) - 1] = '\0';

	for (i = 0; i < thisPtr->num_tracks; ++i)
	{
		if (thisPtr->Track[i]->fccType == streamtypeAUDIO)
		{
			if (index == 0)
			{
				qtt = thisPtr->Track[i];
				break;
			}
			--index;
		}
	}
	if (i == thisPtr->num_tracks)
	{
		SetDlgItemTextA(hDlg, IDC_AUDIO_STREAMNUM, "");
		return;
	}
	SetDlgItemTextA(hDlg, IDC_AUDIO_STREAMNUM, buf);

	wfex = (WAVEFORMATEX *)qtt->pFormat;

/**************************************************************************
 *      Sampling rate, channels:
 **************************************************************************/

	i = _snprintf(buf, sizeof(buf), "%lu Hz", wfex->nSamplesPerSec);
	if (i > 0 && i < sizeof(buf))
	{
		if (wfex->nChannels == 1) {
			_snprintf(buf + i, sizeof(buf) - i, ", Mono");
		} else if (wfex->nChannels == 2) {
			_snprintf(buf + i, sizeof(buf) - i, ", Stereo");
		} else {
			_snprintf(buf + i, sizeof(buf) - i, ", %hu channels", wfex->nChannels);
		}
		
		buf[sizeof(buf) - 1] = '\0';
		SetDlgItemTextA(hDlg, IDC_AUDIO_SAMPLINGRATE, buf);
	}

/**************************************************************************
 *      Actual QuickTime format:
 **************************************************************************/

	switch (qtt->dwFormat)
	{
		case A_AC_3:
			i = _snprintf(buf, sizeof(buf), "Dolby Digital");
			break;

		case A_MS11:
			i = _snprintf(buf, sizeof(buf), "Microsoft IMA ADPCM");
			break;

		case A__MP3:
		case A_MS_U:
			i = _snprintf(buf, sizeof(buf), "MPEG Layer III");
			break;
		case A_MP4A:
			i = _snprintf(buf, sizeof(buf), "MP4 Audio");
			break;
		case A_FL32:
		case A_FL64:
			i = _snprintf(buf, sizeof(buf), "%c%c-bit %s float",
				(qtt->dwFormat >> 16) & 0xFF, qtt->dwFormat >> 24,
				(qtt->iEndianness != 0)? "Intel": "Motorola");
			break;

		case A_IMA4:
			i = _snprintf(buf, sizeof(buf), "IMA 4.0 ADPCM");
			break;

		case A_SAMR:
			i = _snprintf(buf, sizeof(buf), "AMR Narrowband");
			break;
		case A_SAWB:
			i = _snprintf(buf, sizeof(buf), "AMR Wideband");
			break;

		case A_IN24:
		case A_IN32:
		case A_RAW_:
		case A_SOWT:
		case A_TWOS:
			i = _snprintf(buf, sizeof(buf), "%hu-bit %sPCM",
				wfex->wBitsPerSample,
				(wfex->wBitsPerSample >= 16)? ((qtt->iEndianness != 0)? "Intel ": "Motorola "): ""
			);
			break;

		case A_ALAW:
		case A_ULAW:
			i = _snprintf(buf, sizeof(buf), "8-bit %cLaw 2:1",
				(char)(qtt->dwFormat & 255)
			);
			break;

		default:
			i = _snprintf(buf, sizeof(buf), "Unsupported");
			break;
	}

	if (i > 0 && i < sizeof(buf))
	{
		unsigned char fcc[5];
		*(unsigned long *)fcc = qtt->dwFormatQT;

		for (n = 0; n < 4; ++n)
		{
			if (fcc[n] < 32) fcc[n] = 32;
		}
		fcc[4] = 0;
		_snprintf(buf + i, sizeof(buf) - i, " (%s)", (char *)fcc);
	}

	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_AUDIO_QTFORMAT, buf);

/**************************************************************************
 *      Decompressor (ACM tag):
 **************************************************************************/

	if (wfex->wFormatTag == WAVE_FORMAT_PCM)
	{
		SetDlgItemTextA(hDlg, IDC_AUDIO_COMPRESSION_STATIC,
			"VirtualDub format (ACM tag):");

		_snprintf(buf, sizeof(buf), "%hu-bit %sPCM (0x%04X)",
			wfex->wBitsPerSample,
			(wfex->wBitsPerSample > 8)? "Intel ": "",
			wfex->wFormatTag
		);
	}
	else if ((qtt->dwFlags & QTT_UNSUPPORTED) == 0)
	{
		ACMFORMATTAGDETAILSA aftd = { 0 };

		SetDlgItemTextA(hDlg, IDC_AUDIO_COMPRESSION_STATIC,
			"Decompressor (ACM tag):");

		aftd.cbStruct = sizeof(aftd);
		aftd.dwFormatTag = wfex->wFormatTag;

		i = acmFormatTagDetailsA(NULL, &aftd, ACM_FORMATTAGDETAILSF_FORMATTAG);

		if (i == MMSYSERR_NOERROR)
		{
			_snprintf(buf, sizeof(buf),
				"%s (0x%04X)", aftd.szFormatTag, wfex->wFormatTag);
		} else {
			_snprintf(buf, sizeof(buf), "Unknown (0x%04X)", wfex->wFormatTag);
		}
	}
	else
	{
		_snprintf(buf, sizeof(buf), "Unsupported format");
	}

	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_AUDIO_COMPRESSION, buf);

/**************************************************************************
 *      Block layout:
 **************************************************************************/

	_snprintf(buf, sizeof(buf), "%lu blocks", qtt->num_samples);
	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_AUDIO_LAYOUT, buf);

	if (qtt->num_samples > 0)
	{
		char *s;
		double dur;

/**************************************************************************
 *      Length (duration):
 **************************************************************************/

		if (qtt->dwTimeScale != 0)
			dur = (double)qtt->i64Duration / (double)qtt->dwTimeScale;
		else
			dur = 0.0;

		s = time_to_string(dur);
		if (s != NULL)
		{
			_snprintf(buf, sizeof(buf), "%I64u samples (%s)",
				(unsigned __int64)(dur * wfex->nSamplesPerSec + 0.5), s);
			delete[] s;
		}
		else
		{
			_snprintf(buf, sizeof(buf), "%I64u samples (0:00.000)",
				(unsigned __int64)(dur * wfex->nSamplesPerSec + 0.5));
		}

		buf[sizeof(buf) - 1] = '\0';
		SetDlgItemTextA(hDlg, IDC_AUDIO_LENGTH, buf);

/**************************************************************************
 *      Min/avg/max(total) block size:
 **************************************************************************/

		pInfo.dwFrameMinSize[0] = 0xFFFFFFFF;

		for (i = 0; (unsigned long)i < qtt->num_samples; ++i)
		{
			unsigned long dwSize = qtt->sample_list[i].size & (~QT_KEYFRAME);

			if (dwSize < pInfo.dwFrameMinSize[0])
				pInfo.dwFrameMinSize[0] = dwSize;

			if (dwSize > pInfo.dwFrameMaxSize[0])
				pInfo.dwFrameMaxSize[0] = dwSize;
		}

		_snprintf(buf, sizeof(buf), "%lu / %lu / %lu (%I64i)",
			pInfo.dwFrameMinSize[0],
			(unsigned long)((double)qtt->i64TotalBytes
						  / (double)qtt->num_samples + 0.5),
			pInfo.dwFrameMaxSize[0],
			qtt->i64TotalBytes);

		buf[sizeof(buf) - 1] = '\0';
		SetDlgItemTextA(hDlg, IDC_AUDIO_FRAMESIZES, buf);
	}
	else
	{
		// No samples!
		SetDlgItemTextA(hDlg, IDC_AUDIO_LENGTH, "0 samples (0:00)");
		SetDlgItemTextA(hDlg, IDC_AUDIO_FRAMESIZES, "0 / 0 / 0 (0K)");
	}

/**************************************************************************
 *      Data rate:
 **************************************************************************/

	// (a / b) / c  =  a / (b * c)
	// a / (b / c)  =  (a * c) / b

	// kbits_per_sec = (total_bits / Duration_in_seconds) / 1000
	// kbits_per_sec = (total_bits / (dwDuration / dwTimeScale)) / 1000
	// kbits_per_sec = ((total_bits * dwTimeScale) / dwDuration) / 1000
	// kbits_per_sec = (total_bits * dwTimeScale) / (dwDuration * 1000)

	if (qtt->i64Duration > 0)
	{
		unsigned long rate = (unsigned long)(
			(8.0 * qtt->i64TotalBytes * qtt->dwTimeScale)
			/ qtt->i64Duration + 0.5
		);
		_snprintf(buf, sizeof(buf), "%lu bits/sec", rate);
	}
	else
	{
		// Invalid duration!
		_snprintf(buf, sizeof(buf), "0 bits/sec");
	}

	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_AUDIO_DATARATE, buf);
}


void InputFileQT::UpdateDlgVideo(HWND hDlg, int index)
{
	char buf[64];
//	unsigned char fcc[4];
	int i, n;

	DlgStreamInfo pInfo = { 0 };

	const QTSampleInfo     *si;
	const BITMAPINFOHEADER *bmih;
	const QTTrack          *qtt;

	InputFileQT *thisPtr =
		(InputFileQT *)GetWindowLongPtrA(hDlg, DWLP_USER);

	for (i = 0; i < thisPtr->num_tracks; ++i)
	{
		if (thisPtr->Track[i]->fccType == streamtypeVIDEO)
		{
			if (index == 0)
			{
				qtt = thisPtr->Track[i];
				break;
			}
			--index;
		}
	}
	if (i == thisPtr->num_tracks) return;

/**************************************************************************
 *      Frame size, average fps:
 **************************************************************************/

	bmih = (BITMAPINFOHEADER *)qtt->pFormat;

	if (qtt->dwScale != 0)
	{
		_snprintf(buf, sizeof(buf), "%dx%d, %.3f fps",
			bmih->biWidth, bmih->biHeight,
			(double)qtt->dwRate / (double)qtt->dwScale
		);
	}
	else
	{
		// Invalid dwRate/dwScale!
		_snprintf(buf, sizeof(buf), "%dx%d, 0.000 fps",
			bmih->biWidth, bmih->biHeight);
	}

	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_VIDEO_FORMAT, buf);

/**************************************************************************
 *      Pixel aspect ratio:
 **************************************************************************/

	if (qtt->hSpacing != 0 && qtt->vSpacing != 0)
	{
		i = (int)((1000.0 * qtt->hSpacing) / qtt->vSpacing + 0.5);

		_snprintf(buf, sizeof(buf), "%lu / %lu = %i.%03i",
			qtt->hSpacing, qtt->vSpacing, i / 1000, i % 1000);
	}
	else
	{
		_snprintf(buf, sizeof(buf), "(not indicated) 1 / 1 = 1.000");
	}

	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_VIDEO_ASPECT_RATIO, buf);

/**************************************************************************
 *      Perceived frames (duration):
 **************************************************************************/

	if (qtt->dwRate != 0)
	{
		char *s = time_to_string(
			((double)qtt->perceived_samples * (double)qtt->dwScale)
			/ (double)qtt->dwRate
		);

		if (s != NULL)
		{
			_snprintf(buf, sizeof(buf), "%ld (%s)", qtt->perceived_samples, s);
			delete[] s;
		}
		else
		{
			_snprintf(buf, sizeof(buf), "%ld (0:00.000)", qtt->perceived_samples);
		}
	}
	else
	{
		// Invalid dwRate/dwScale!
		_snprintf(buf, sizeof(buf), "%ld (0:00)", qtt->perceived_samples);
	}

	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_VIDEO_NUMFRAMES, buf);

/**************************************************************************
 *      Actual QuickTime format:
 **************************************************************************/

	switch (qtt->dwFormat)
	{
		case V_AVC1:
			i = _snprintf(buf, sizeof(buf), "MPEG-4 AVC");
			break;
		case V_CVID:
			i = _snprintf(buf, sizeof(buf), "Cinepak");
			break;
		case V_DVC_:
			i = _snprintf(buf, sizeof(buf), "QuickTime NTSC DV-25");
			break;
		case V_DVCP:
			i = _snprintf(buf, sizeof(buf), "QuickTime PAL DV-25");
			break;
		case V_H263:
			i = _snprintf(buf, sizeof(buf), "H.263");
			break;
		case V_IV41:
			i = _snprintf(buf, sizeof(buf), "Indeo Video 4.1");
			break;
		case V_JPEG:
			i = _snprintf(buf, sizeof(buf), "JPEG");
			break;
		case V_MJPA:
		case V_MJPB:
		case V_MJPG:
			i = _snprintf(buf, sizeof(buf), "Motion JPEG");
			break;
		case V_MP4V:
			i = _snprintf(buf, sizeof(buf), "MP4 Video");
			break;
		case V_RLE_:
			i = _snprintf(buf, sizeof(buf), "Run-Length Encoded");
			break;
		case V_RPZA:
			i = _snprintf(buf, sizeof(buf), "Apple Roadpizza");
			break;
		case V_S263:
			i = _snprintf(buf, sizeof(buf), "3GPP H.263");
			break;
		case V_SVQ1:
		case V_SVQ3:
			i = _snprintf(buf, sizeof(buf), "Sorenson Video v%c", qtt->dwFormat >> 24);
			break;
		case V_VC_1:
			i = _snprintf(buf, sizeof(buf), "SMPTE VC-1");
			break;
		default:
			i = _snprintf(buf, sizeof(buf), "Unrecognized format");
			break;
	}

	if (i > 0 && i < sizeof(buf))
	{
		unsigned char fcc[5];
		*(unsigned long *)fcc = qtt->dwFormatQT;

		for (n = 0; n < 4; ++n)
		{
			if (fcc[n] < 32) fcc[n] = 32;
		}
		fcc[4] = 0;
		_snprintf(buf + i, sizeof(buf) - i, " (%s)", (char *)fcc);
	}

	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_VIDEO_QTFORMAT, buf);

/**************************************************************************
 *      Decompressor (fourcc):
 **************************************************************************/

	if (thisPtr->pOptions->DeferDecoding())
	{
		i = _snprintf(buf, sizeof(buf), "VirtualDub");
	}
	else
	{
		if (thisPtr->vCodec != NULL)
		{
			ICINFO ici;

			ici.dwSize = sizeof(ici);
			if (ICGetInfo(thisPtr->vCodec, &ici, sizeof(ici))
				>= FIELD_OFFSET(ICINFO, szDriver))
			{
				i = _snprintf(buf, sizeof(buf), "%ls", ici.szDescription);
			}
			else
			{
				i = _snprintf(buf, sizeof(buf), "Unknown");
			}
		}
		else
		{
			i = _snprintf(buf, sizeof(buf), "Missing codec");
		}
	}

	if (i > 0 && i < sizeof(buf))
	{
		unsigned char fcc[5];
		*(unsigned long *)fcc = bmih->biCompression;

		for (n = 0; n < 4; ++n)
		{
			if (fcc[n] < 32) fcc[n] = 32;
		}
		fcc[4] = 0;
		_snprintf(buf + i, sizeof(buf) - i, " (%s)", (char *)fcc);
	}

	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_VIDEO_COMPRESSION, buf);

/**************************************************************************
 *      Number of key frames:
 **************************************************************************/

	if (qtt->num_samples > 0)
	{
		pInfo.dwFrameMinSize[0] = 0xFFFFFFFF;
		pInfo.dwFrameMinSize[1] = 0xFFFFFFFF;
	}

	for (i = 0; (unsigned long)i < qtt->num_samples; ++i)
	{
		si = &(qtt->sample_list[i]);
		int iFrameType = (si->size & QT_KEYFRAME)? 0: 1;
		unsigned long dwSize = si->size & (~QT_KEYFRAME);

		++pInfo.dwFrameCount[iFrameType];
		pInfo.i64FrameTotalSize[iFrameType] += dwSize;

		if (dwSize < pInfo.dwFrameMinSize[iFrameType])
			pInfo.dwFrameMinSize[iFrameType] = dwSize;

		if (dwSize > pInfo.dwFrameMaxSize[iFrameType])
			pInfo.dwFrameMaxSize[iFrameType] = dwSize;
	}

	_snprintf(buf, sizeof(buf), "%lu", pInfo.dwFrameCount[0]);

	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_VIDEO_NUMKEYFRAMES, buf);

/**************************************************************************
 *      Min/avg/max(total) key frame size:
 **************************************************************************/

	if (pInfo.dwFrameCount[0] != 0)
	{
		_snprintf(buf, sizeof(buf), "%lu / %lu / %lu (%I64i)",
			pInfo.dwFrameMinSize[0],
			(unsigned long)((double)pInfo.i64FrameTotalSize[0] / (double)pInfo.dwFrameCount[0] + 0.5),
			pInfo.dwFrameMaxSize[0],
			pInfo.i64FrameTotalSize[0]);
	}
	else
	{
		_snprintf(buf, sizeof(buf), "(no key frames)");
	}

	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_VIDEO_KEYFRAMESIZES, buf);

/**************************************************************************
 *      Min/avg/max(total) delta frame size:
 **************************************************************************/

	if (pInfo.dwFrameCount[1] != 0)
	{
		_snprintf(buf, sizeof(buf), "%lu / %lu / %lu (%I64i)",
			pInfo.dwFrameMinSize[1],
			(unsigned long)((double)pInfo.i64FrameTotalSize[1] / (double)pInfo.dwFrameCount[1] + 0.5),
			pInfo.dwFrameMaxSize[1],
			pInfo.i64FrameTotalSize[1]);
	}
	else
	{
		_snprintf(buf, sizeof(buf), "(no delta frames)");
	}

	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_VIDEO_NONKEYFRAMESIZES, buf);

/**************************************************************************
 *      Data rate:
 **************************************************************************/

	if (qtt->i64Duration > 0)
	{
		unsigned long rate = (unsigned long)(
			(8.0 * qtt->i64TotalBytes * qtt->dwTimeScale)
			/ qtt->i64Duration + 0.5
		);
		_snprintf(buf, sizeof(buf), "%lu bits/sec", rate);
	}
	else
	{
		// Invalid duration!
		_snprintf(buf, sizeof(buf), "0 bits/sec");
	}

	buf[sizeof(buf) - 1] = '\0';
	SetDlgItemTextA(hDlg, IDC_VIDEO_DATARATE, buf);
}


void InputFileQT::UpdateDlgFile(HWND hDlg)
{
	int i;

/**************************************************************************
 *      Major brand:
 **************************************************************************/

	InputFileQT *thisPtr =
		(InputFileQT *)GetWindowLongPtrA(hDlg, DWLP_USER);

	if (thisPtr->major_brand != 0)
	{
		char buf[64];
		unsigned char ftyp[4];

		*(unsigned long *)ftyp = thisPtr->major_brand;
		for (i = 0; i < 4; ++i)
		{
			if (ftyp[i] < 32) ftyp[i] = 32;
		}

		if (thisPtr->compat_brand != 0)
		{
			unsigned char compat[4];

			*(unsigned long *)compat = thisPtr->compat_brand;
			for (i = 0; i < 4; ++i)
			{
				if (compat[i] < 32) compat[i] = 32;
			}

			_snprintf(buf, sizeof(buf), "'%c%c%c%c' (via '%c%c%c%c' compatibility)",
				ftyp[0], ftyp[1], ftyp[2], ftyp[3],
				compat[0], compat[1], compat[2], compat[3]);
		}
		else
		{
			_snprintf(buf, sizeof(buf), "'%c%c%c%c'",
				ftyp[0], ftyp[1], ftyp[2], ftyp[3]);
		}

		buf[sizeof(buf) - 1] = '\0';
		SetDlgItemTextA(hDlg, IDC_MAJOR_BRAND, buf);
	}
	else
	{
		SetDlgItemTextA(hDlg, IDC_MAJOR_BRAND, "(not indicated)");
	}
}


INT_PTR InputFileQT::InfoDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	static int iAudioTracks;
	InputFileQT *thisPtr;
	int i;

	switch (uMsg) {

    case WM_INITDIALOG:
		SetWindowLongPtrA(hDlg, DWLP_USER, lParam);
		thisPtr = (InputFileQT *)lParam;

		InputOptionsQT::InitDialogTitle(hDlg, " - File Information");

		// Show the "View Import Log..." button?
		if (thisPtr->log.Len() > 0)
		{
			HWND hwndLog = GetDlgItem(hDlg, IDC_IMPORT_LOG);
			EnableWindow(hwndLog, TRUE);
			ShowWindow(hwndLog, SW_SHOW);
		}

		// Update QuickTime file info
		UpdateDlgFile(hDlg);

		// Update video track info
		UpdateDlgVideo(hDlg, 0);

		// Update audio track info

		iAudioTracks = 0;

		for (i = 0; i < thisPtr->num_tracks; ++i)
		{
			if (thisPtr->Track[i]->fccType == streamtypeAUDIO)
			{
				++iAudioTracks;
			}
		}

		if (iAudioTracks > 1)
		{
			HWND hwndSpin = GetDlgItem(hDlg, IDC_AUDIO_SPIN);
			EnableWindow(hwndSpin, TRUE);
			SendMessageA(hwndSpin, UDM_SETRANGE, 0,
				(LPARAM)MAKELONG(iAudioTracks - 1, 0));
			SendMessageA(hwndSpin, UDM_SETPOS, 0, 0);
			ShowWindow(hwndSpin, SW_SHOW);
		}

		UpdateDlgAudio(hDlg, 0);
		return TRUE;

	case WM_COMMAND:
		// Note: "IDCANCEL" is the [X] button
		// Return TRUE if we process the message

		if (LOWORD(wParam) == IDC_IMPORT_LOG)
		{
			thisPtr = (InputFileQT *)GetWindowLongPtrA(hDlg, DWLP_USER);
			DialogBoxParamA(g_hModule, MAKEINTRESOURCEA(IDD_IMPORT_LOG), hDlg, LogDlgProc, (LPARAM)thisPtr);
			return TRUE;
		}

		if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL)
		{
			EndDialog(hDlg, TRUE);
			return TRUE;
		}
		break;

	case WM_NOTIFY:
	{
		const NMUPDOWN *nmud = (NMUPDOWN *)lParam;

		if (nmud->hdr.idFrom == IDC_AUDIO_SPIN && nmud->hdr.code == UDN_DELTAPOS)
		{
			thisPtr = (InputFileQT *)GetWindowLongPtrA(hDlg, DWLP_USER);

			i = nmud->iPos + nmud->iDelta;

			// The UDS_WRAP style does wrap the position,
			// but only after we get this message (aargh).
			// Therefore we have to wrap it manually...

			if (i < 0) i = iAudioTracks - 1;
			if (i >= iAudioTracks) i = 0;

			UpdateDlgAudio(hDlg, i);
		}

		break;
	}

    }

    return FALSE;
}

