#ifndef _INPUTFILEQT_H_
#define _INPUTFILEQT_H_


//#define SHOW_PARSE_DIALOG

class InputFileQT : public vdxunknown<IVDXInputFile> {

private:
	WinFile     mFile;

	QTTrack*    Track[MAX_TRACKS];
	int         num_tracks;
	ErrorLog    log;

	const       VDXInputDriverContext& mContext;

	// ftyp atom
	unsigned long   major_brand;
	unsigned long   minor_version;
	unsigned long   compat_brand;

#ifdef SHOW_PARSE_DIALOG
	bool    fAbort;
	HWND    hwndProgress;
	static  BOOL CALLBACK FindContextWindow(HWND hwnd, LPARAM lParam);
	static  INT_PTR CALLBACK ParseDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
	bool    guiDlgMessageLoop(HWND hDlg);
#endif

	bool    NextAtom(QTAtom& atom);
	bool    NextMoovAtom(QTAtom& atom);

	unsigned char*  moov_bytes;
	unsigned long   moov_size;
	unsigned long   moov_pos;
	int             moov_read(void *dst, int bytes);

	int TransformAVCConfRec(const unsigned char *src, int srclen, unsigned char *dst, int &len) const;

	void    parse_esds_name(int max);
	void    parse_esds(SDExtensions& ex, __int64 limit);
	void    parse_wave(SDExtensions& ex, __int64 limit);
	bool    parse_exts(SDExtensions& ex, __int64 limit);

	unsigned long   RemapDefault(unsigned long dwFormat, unsigned long dwObjectID) const;

	bool    parse_vids(QTTrack& qtt, int size);
	bool    parse_auds(QTTrack& qtt, int size);

	void    parse_stbl(QTTrack& qtt, __int64 limit);
	void    parse_minf(QTTrack& qtt, __int64 limit);
	void    parse_mdia(QTTrack& qtt, __int64 limit);
	void    parse_trak(__int64 limit);
	bool    parse_moov(const QTAtom& moov_atom);

	unsigned long   preparse_fmt(__int64 limit);
	const wchar_t*  FileTitle(const wchar_t *szFile) const;

	bool    IsSupported_ftyp(unsigned long ftyp) const;
	bool    PostInit();

	static char*            time_to_string(double seconds);
	static void             UpdateDlgAudio(HWND hDlg, int index);
	static inline void      UpdateDlgVideo(HWND hDlg, int index);
	static inline void      UpdateDlgFile (HWND hDlg);

	static INT_PTR CALLBACK InfoDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
	static INT_PTR CALLBACK LogDlgProc (HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

public:
	InputFileQT(const VDXInputDriverContext& context);
	~InputFileQT();

	// IVDXInputFile methods
	bool    VDXAPIENTRY PromptForOptions(VDXHWND, IVDXInputOptions **ppOptions);
	bool    VDXAPIENTRY CreateOptions(const void *buf, uint32 len, IVDXInputOptions **ppOptions);

	void    VDXAPIENTRY Init(const wchar_t *path, IVDXInputOptions *options);
	bool    VDXAPIENTRY Append(const wchar_t *path);

	void    VDXAPIENTRY DisplayInfo(VDXHWND hwndParent);

	bool    VDXAPIENTRY GetVideoSource(int index, IVDXVideoSource **ppVS);
	bool    VDXAPIENTRY GetAudioSource(int index, IVDXAudioSource **ppAS);

	// InputFileQT methods
	int  ReadWholeSample(void *lpBuffer, const QTSampleInfo& si);
	int  ReadRawStream(void *lpBuffer, __int64 bytepos, int bytes, QTTrack& qtt);

	unsigned long   LowercaseFormat(unsigned long tag) const;
	unsigned long   UppercaseFormat(unsigned long tag) const;

	// Video codec handle, set by VideoDecoderQT
	HIC  vCodec;

	// Options if any
	InputOptionsQT *pOptions;
};


#endif  // _INPUTFILEQT_H_

