///////////////////////////////////////////////////////////////////////
// QuickTime plugin for VirtualDub 1.8.8+
// Copyright (C) 2010-2012 fccHandler
// Copyright (C) 1998-2012 Avery Lee
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
///////////////////////////////////////////////////////////////////////

// IVDXInputOptions implementation

#if defined(_MSC_VER) && (_MSC_VER >= 1400)
	#define _CRT_SECURE_NO_WARNINGS
#endif

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdio.h>
#include <crtdbg.h>

#include "Unknown.h"
#include "vd2\plugin\vdinputdriver.h"
#include "InputOptionsQT.h"
#include "Resource.h"

extern "C" const VDXPluginInfo *const * VDXAPIENTRY VDGetPluginInfo();

static const char QtRegKey[] = "Software\\fccHandler\\Plugins\\QuickTime";

extern HMODULE g_hModule;	// InputFileQT.cpp


InputOptionsQT::InputOptionsQT()
	: map (NULL)
{
_RPT1(_CRT_WARN, "InputOptionsQT (0x%p) created\n", this);
	ReadMap(this);
}


InputOptionsQT::~InputOptionsQT()
{
_RPT1(_CRT_WARN, "InputOptionsQT (0x%p) destroyed\n", this);
	delete[] map;
}


void InputOptionsQT::ReadMap(InputOptionsQT *thisPtr)
{
	HKEY hKey;

	// Default options
	thisPtr->opts.len   = sizeof(OptionsQT);
	thisPtr->opts.flags = 0;

	delete[] thisPtr->map;
	thisPtr->map   = NULL;
	thisPtr->count = 0;

	if (RegOpenKeyExA(HKEY_CURRENT_USER, QtRegKey, 0, KEY_READ, &hKey) == ERROR_SUCCESS)
	{
		LONG  lResult;
		DWORD cbData = sizeof(OptionsQT);

		RegQueryValueExA(hKey, "Options", NULL, NULL,
			(LPBYTE)&thisPtr->opts, &cbData);

		thisPtr->opts.len = sizeof(OptionsQT);

		// Note: Version 1.5 stored "flags" as a bool, so for
		// compatibility do not use any of its upper 24 bits!
		thisPtr->opts.flags &= QT_OPTS_MASK;

		lResult = RegQueryValueExA(hKey, "FourccMap", NULL, NULL, NULL, &cbData);

		if (lResult == ERROR_SUCCESS && cbData >= sizeof(FourccEntry))
		{
			thisPtr->map = new char[cbData];
			if (thisPtr->map != NULL)
			{
				lResult = RegQueryValueExA(hKey, "FourccMap", NULL, NULL, (LPBYTE)thisPtr->map, &cbData);
				if (lResult == ERROR_SUCCESS)
				{
					thisPtr->count = cbData / sizeof(FourccEntry);
					thisPtr->opts.len += thisPtr->count * sizeof(FourccEntry);
				}
				else
				{
					delete[] thisPtr->map;
					thisPtr->map = NULL;
				}
			}
		}
		RegCloseKey(hKey);
	}
}


void InputOptionsQT::WriteMap(InputOptionsQT *thisPtr)
{
	HKEY hKey;

	if (RegCreateKeyExA(HKEY_CURRENT_USER, QtRegKey,
		0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS,
		NULL, &hKey, NULL) == ERROR_SUCCESS)
	{
		thisPtr->opts.len = sizeof(OptionsQT);

		RegSetValueExA(hKey, "Options", 0, REG_BINARY,
			(CONST BYTE *)&thisPtr->opts, sizeof(OptionsQT)
		);

		if (thisPtr->count > 0)
		{
			DWORD cbData = thisPtr->count * sizeof(FourccEntry);
			thisPtr->opts.len += cbData;

			RegSetValueExA(hKey, "FourccMap", 0, REG_BINARY,
				(CONST BYTE *)thisPtr->map, cbData);
		}
		else
		{
			RegDeleteValueA(hKey, "FourccMap");
		}

		RegCloseKey(hKey);
	}
}


unsigned long InputOptionsQT::RemapFourcc(unsigned long fourcc)
{
	if (count > 0)
	{
		const FourccEntry *fcc = (FourccEntry *)map;

		for (int i = 0; i < count; ++i)
		{
			if (fcc[i].InFourcc == fourcc)
			{
				fourcc = fcc[i].OutFourcc;
				break;
			}
		}
	}

	return fourcc;
}


uint32 VDXAPIENTRY InputOptionsQT::Write(void *buf, uint32 buflen)
{
	if (buf != NULL && buflen >= opts.len)
	{
		memcpy(buf, &opts, sizeof(OptionsQT));
		if (count > 0)
		{
			memcpy((char *)buf + sizeof(OptionsQT), map, count * sizeof(FourccEntry));
		}
	}
	return opts.len;
}


void InputOptionsQT::Load(const void *buf, int len)
{
	opts.len = sizeof(OptionsQT);
	opts.flags = 0;

	delete[] map;
	map = NULL;
	count = 0;

	if (buf != NULL && len >= sizeof(OptionsQT))
	{
		memcpy(&opts, buf, sizeof(OptionsQT));

		buf = (char *)buf + sizeof(OptionsQT);
		len -= sizeof(OptionsQT);

		len /= sizeof(FourccEntry);
		if (len > 0)
		{
			map = new char[len * sizeof(FourccEntry)];
			if (map != NULL)
			{
				memcpy(map, buf, len * sizeof(FourccEntry));
				count = len;
			}
		}
	}
}


void InputOptionsQT::UpdateButtons(HWND hDlg)
{
	unsigned char buf[16];
	BOOL bEnableAdd;
	BOOL bEnableRemove;
	int  i;

	unsigned char *f1 = (unsigned char *)(buf + 0);
	unsigned char *f2 = (unsigned char *)(buf + 8);

	HWND hEdit1   = GetDlgItem(hDlg, IDC_FOURCC_IN);
	HWND hEdit2   = GetDlgItem(hDlg, IDC_FOURCC_OUT);
	HWND hList    = GetDlgItem(hDlg, IDC_FOURCC_LIST);

	bEnableAdd    = FALSE;
	bEnableRemove = FALSE;

	i = (int)SendMessageA(hList, LB_GETCOUNT, 0, 0);
	if (i > 0)
	{
		int sel = (int)SendMessageA(hList, LB_GETCURSEL, 0, 0);
		if ((unsigned int)sel < (unsigned int)i) bEnableRemove = TRUE;
	}
	
	i = GetWindowTextA(hEdit1, (LPSTR)f1, 7);
	if (i == 4 && f1[0] >= 32 && f1[1] >= 32 && f1[2] >= 32 && f1[3] >= 32)
	{
		i = GetWindowTextA(hEdit2, (LPSTR)f2, 7);
		if (i == 4 && f2[0] >= 32 && f2[1] >= 32 && f2[2] >= 32 && f2[3] >= 32)
		{
			bEnableAdd = TRUE;
		}
	}

	EnableWindow(GetDlgItem(hDlg, IDC_FOURCC_ADD), bEnableAdd);
	EnableWindow(GetDlgItem(hDlg, IDC_FOURCC_REMOVE), bEnableRemove);
}


void InputOptionsQT::AddClick(HWND hDlg)
{
	char buf[128];
	int i, iCount;

	unsigned char *f1  = (unsigned char *)(buf +  0);
	unsigned char *f2  = (unsigned char *)(buf +  8);
	char *out = buf + 16;

	HWND hList  = GetDlgItem(hDlg, IDC_FOURCC_LIST);
	HWND hEdit1 = GetDlgItem(hDlg, IDC_FOURCC_IN);
	HWND hEdit2 = GetDlgItem(hDlg, IDC_FOURCC_OUT);

	iCount = (int)SendMessageA(hList, LB_GETCOUNT, 0, 0);
	
	i = GetWindowTextA(hEdit1, (LPSTR)f1, 7);
	if (i != 4 || f1[0] < 32 || f1[1] < 32 || f1[2] < 32 || f1[3] < 32) return;
	f1[4] = 0;

	i = GetWindowTextA(hEdit2, (LPSTR)f2, 7);
	if (i != 4 || f2[0] < 32 || f2[1] < 32 || f2[2] < 32 || f2[3] < 32) return;
	f2[4] = 0;

	if (iCount > 0)
	{
		for (i = 0; i < iCount; ++i)
		{
			if (SendMessageA(hList, LB_GETTEXT, (WPARAM)i, (LPARAM)out) == 20)
			{
				if (*(DWORD *)f1 == *(DWORD *)(out + 5))
				{
					*(DWORD *)f2 =  *(DWORD *)(out + 15);
					sprintf(out, "Cannot add '%s' because it is already mapped to '%s'", f1, f2);
					MessageBoxA(hDlg, out, NULL, MB_OK | MB_ICONSTOP);
					return;
				}
			}
		}
	}
	else
	{
		iCount = 0;
	}

	if (*(DWORD *)f1 == *(DWORD *)f2)
	{
		sprintf(out, "Cannot add mapping '%s' to '%s' because it has no effect.", f1, f2);
		MessageBoxA(hDlg, out, NULL, MB_OK | MB_ICONSTOP);
		return;
	}

	sprintf(out, "Map '%s' to '%s'", f1, f2);
	SendMessageA(hList, LB_ADDSTRING, 0, (LPARAM)out);
	SendMessageA(hList, LB_SETCURSEL, (WPARAM)iCount, 0);

	SetWindowTextA(hEdit1, "");
	SetWindowTextA(hEdit2, "");
	UpdateButtons(hDlg);
}


void InputOptionsQT::RemoveClick(HWND hDlg)
{
	int i;
	HWND hList = GetDlgItem(hDlg, IDC_FOURCC_LIST);

	i = (int)SendMessageA(hList, LB_GETCOUNT, 0, 0);
	if (i > 0)
	{
		int sel = (int)SendMessageA(hList, LB_GETCURSEL, 0, 0);
		if ((unsigned int)sel < (unsigned int)i)
		{
			char buf[24];

			HWND hEdit1 = GetDlgItem(hDlg, IDC_FOURCC_IN);
			HWND hEdit2 = GetDlgItem(hDlg, IDC_FOURCC_OUT);

			// If both edit boxes are empty...
			if (GetWindowTextA(hEdit1, buf, sizeof(buf) - 1) == 0 &&
			    GetWindowTextA(hEdit2, buf, sizeof(buf) - 1) == 0 )
			{
				if (SendMessageA(hList, LB_GETTEXT, (WPARAM)sel, (LPARAM)buf) == 20)
				{
					// Copy the deleted FOURCCs into the edit boxes
					*(DWORD *)(buf + 0) = *(DWORD *)(buf +  5);
					*(DWORD *)(buf + 8) = *(DWORD *)(buf + 15);

					buf[4]  = '\0';
					buf[12] = '\0';

					SetWindowTextA(GetDlgItem(hDlg, IDC_FOURCC_IN ), buf + 0);
					SetWindowTextA(GetDlgItem(hDlg, IDC_FOURCC_OUT), buf + 8);
				}
			}

			SendMessageA(hList, LB_DELETESTRING, (WPARAM)sel, 0);
			if (i > 1 && sel == (i - 1))
			{
				--sel;
				SendMessageA(hList, LB_SETCURSEL, (WPARAM)sel, 0);
			}
		}
	}

	UpdateButtons(hDlg);
}


void InputOptionsQT::InitFourccList(HWND hDlg, const InputOptionsQT *thisPtr)
{
	HWND hList = GetDlgItem(hDlg, IDC_FOURCC_LIST);
	if (hList != NULL && thisPtr->count > 0)
	{
		char buf[40];
		int i;

		const FourccEntry *fcc = (FourccEntry *)thisPtr->map;

		unsigned char *f1 = (unsigned char *)(buf + 0);
		unsigned char *f2 = (unsigned char *)(buf + 8);
		char *out = buf + 16;

		for (i = 0; i < thisPtr->count; ++i)
		{
			*(DWORD *)f1 = fcc[i].InFourcc;
			f1[4] = 0;

			*(DWORD *)f2 = fcc[i].OutFourcc;
			f2[4] = 0;
			
			for (int k = 0; k < 4; ++k)
			{
				if (f1[k] < 32) f1[k] = 32;
				if (f2[k] < 32) f2[k] = 32;
			}
			
			sprintf(out, "Map '%s' to '%s'", f1, f2);
			SendMessageA(hList, LB_ADDSTRING, 0, (LPARAM)out);
		}

		SendMessageA(hList, LB_SETCURSEL, 0, 0);
	}
}


void InputOptionsQT::OkClick(HWND hDlg, InputOptionsQT *thisPtr, bool bSave)
{
	int i;
	HWND hList = GetDlgItem(hDlg, IDC_FOURCC_LIST);

	delete[] thisPtr->map;

	thisPtr->opts.len   = sizeof(OptionsQT);
	thisPtr->opts.flags = 0;
	thisPtr->map        = NULL;
	thisPtr->count      = 0;

	if (IsDlgButtonChecked(hDlg, IDC_DEFER_DECODING) == BST_CHECKED)
	{
		thisPtr->opts.flags |= QT_OPTS_DEFER_DECODING;
	}
	if (IsDlgButtonChecked(hDlg, IDC_ADD_ADTS) == BST_CHECKED)
	{
		thisPtr->opts.flags |= QT_OPTS_ADD_ADTS;
	}
	if (IsDlgButtonChecked(hDlg, IDC_ADD_AMR) == BST_CHECKED)
	{
		thisPtr->opts.flags |= QT_OPTS_ADD_AMR;
	}

	i = (int)SendMessageA(hList, LB_GETCOUNT, 0, 0);
	if (i > 0)
	{
		thisPtr->map = new char[i * sizeof(FourccEntry)];
		if (thisPtr->map != NULL)
		{
			char buf[24] = { 0 };

			FourccEntry *fcc = (FourccEntry *)thisPtr->map;
			thisPtr->count = i;
			thisPtr->opts.len += i * sizeof(FourccEntry);

			for (i = 0; i < thisPtr->count; ++i)
			{
				SendMessageA(hList, LB_GETTEXT, (WPARAM)i, (LPARAM)buf);
				// "Map 'abcd' to 'abcd'"
				//  01234567890123456789
				fcc[i].InFourcc  = *(DWORD *)(buf +  5);
				fcc[i].OutFourcc = *(DWORD *)(buf + 15);
			}
		}
	}
	
	if (bSave) WriteMap(thisPtr);
}


void InputOptionsQT::InitDialogTitle(HWND hDlg, char *pTitle)
{
	char buf[64];
	int i;

	uint32 ver = (*VDGetPluginInfo())->mVersion;

	i = _snprintf(buf, sizeof(buf), "QuickTime Plugin v%i.%i",
		(ver >> 24) & 0xFF, (ver >> 16) & 0xFF);

	if (i > 0 && i < sizeof(buf) && pTitle != NULL)
	{
		_snprintf(buf + i, sizeof(buf) - i, "%s", pTitle);
	}

	buf[sizeof(buf) - 1] = '\0';
	SetWindowTextA(hDlg, buf);
}


INT_PTR CALLBACK InputOptionsQT::OptionsDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	InputOptionsQT *thisPtr;

	switch (message) {

	case WM_INITDIALOG:
		SetWindowLongPtrA(hDlg, DWLP_USER, lParam);
		thisPtr = (InputOptionsQT *)lParam;
		InitDialogTitle(hDlg, " - Extended Options");
		InitFourccList(hDlg, thisPtr);
		UpdateButtons(hDlg);
		CheckDlgButton(hDlg, IDC_DEFER_DECODING,
			(thisPtr->opts.flags & QT_OPTS_DEFER_DECODING)? BST_CHECKED: BST_UNCHECKED);
		CheckDlgButton(hDlg, IDC_ADD_ADTS,
			(thisPtr->opts.flags & QT_OPTS_ADD_ADTS)? BST_CHECKED: BST_UNCHECKED);
		CheckDlgButton(hDlg, IDC_ADD_AMR,
			(thisPtr->opts.flags & QT_OPTS_ADD_AMR)? BST_CHECKED: BST_UNCHECKED);
		return TRUE;

	case WM_COMMAND:
	{
		WORD wChosen = LOWORD(wParam);
		switch (wChosen)
		{
			case IDC_FOURCC_IN:
			case IDC_FOURCC_OUT:
				UpdateButtons(hDlg);
				return TRUE;

			case IDC_FOURCC_ADD:
				AddClick(hDlg);
				return TRUE;

			case IDC_FOURCC_REMOVE:
				RemoveClick(hDlg);
				return TRUE;

			case IDOK:
			case IDC_SAVE:
				thisPtr = (InputOptionsQT *)GetWindowLongPtrA(hDlg, DWLP_USER);
				OkClick(hDlg, thisPtr, wChosen == IDC_SAVE);
				// fall through
			case IDCANCEL:
				EndDialog(hDlg, (wChosen == IDCANCEL)? FALSE: TRUE);
				return TRUE;
		}
		break;
	}

	default:
		break;
	}

	return FALSE;
}

void InputOptionsQT::OptionsDlg(HWND hwndParent)
{
	DialogBoxParamA(g_hModule, MAKEINTRESOURCEA(IDD_OPTIONS),
		hwndParent, OptionsDlgProc, (LPARAM)this);
}

