#ifndef _INPUTOPTIONSQT_H_
#define _INPUTOPTIONSQT_H_

typedef struct {
	DWORD InFourcc;
	DWORD OutFourcc;
} FourccEntry;

class InputOptionsQT : public vdxunknown<IVDXInputOptions>
{
private:
	char*  map;
	int    count;

	static void ReadMap (InputOptionsQT *thisPtr);
	static void WriteMap(InputOptionsQT *thisPtr);

	static void UpdateButtons(HWND hDlg);
	static void AddClick(HWND hDlg);
	static void RemoveClick(HWND hDlg);
	static void OkClick(HWND hDlg, InputOptionsQT *thisPtr, bool bSave);
	static void InitFourccList(HWND hDlg, const InputOptionsQT *thisPtr);
	static INT_PTR CALLBACK OptionsDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);

public:
	InputOptionsQT();
	~InputOptionsQT();

	// flags bits
	enum {
		QT_OPTS_DEFER_DECODING = 0x01,
		QT_OPTS_ADD_ADTS       = 0x02,
		QT_OPTS_ADD_AMR        = 0x04,
		QT_OPTS_MASK           = 0xFF
	};

	uint32 VDXAPIENTRY Write(void *buf, uint32 buflen);
	void OptionsDlg(HWND hwndParent);

	// Note: Version 1.5 stored "flags" as a bool, so for
	// compatibility do not use any of its upper 24 bits!
	// (See	QT_OPTS_MASK above)

	struct OptionsQT {
		uint32  len;
		int     flags;
	} opts;

	static   void InitDialogTitle(HWND hDlg, char *pTitle);
	unsigned long RemapFourcc(unsigned long fourcc);

	void     Load(const void *buf, int len);
	bool     DeferDecoding()  const { return ((opts.flags & QT_OPTS_DEFER_DECODING) != 0); }
	bool     AddADTSHeaders() const { return ((opts.flags & QT_OPTS_ADD_ADTS      ) != 0); }
	bool     AddAMRHeader()   const { return ((opts.flags & QT_OPTS_ADD_AMR       ) != 0); }
};

#endif // _INPUTOPTIONSQT_H_

