#ifndef _QTTRACK_H_
#define _QTTRACK_H_


#define MAX_TRACKS 128

#define MAX_VIDEO_WIDTH  1920
#define MAX_VIDEO_HEIGHT 1088

// Flags for the dwFlags member
#define QTT_HAS_VSB     1	// Audio track has variable-size blocks
#define QTT_ADD_ADTS    2	// Add an ADTS header to each AAC frame
#define QTT_ADD_AMR     4	// Add an AMR header to AMR audio streams
#define QTT_UNSUPPORTED 8   // Unsupported track


class QTTrack {

public:
// trak atom
	unsigned long       dwTrackID;

// mdia atom
	unsigned long       dwTimeScale;
	__int64             i64Duration;
	unsigned long       dwRate;
	unsigned long       dwScale;

// hdlr atom
	unsigned long       fccType;

// stsd atom
	unsigned long       dwFormatQT;     // original fourcc
	unsigned long       dwFormat;       // lowercase fourcc
	unsigned char*      pFormat;
	long                lFormatLen;
	unsigned long       dwSamplesPerPacket;
	unsigned long       dwBytesPerPacket;
	unsigned long       dwBytesPerFrame;
	unsigned long       dwBytesPerSample;

// enda atom
	int                 iEndianness;

// pasp atom
	unsigned long       hSpacing;
	unsigned long       vSpacing;

// stsc atom
	QTChunk*            chunk_info;
	unsigned long       num_chunk_infos;

// stco atom
	unsigned long*      chunk_offset32;
	__int64*            chunk_offset64;
	unsigned long       num_chunk_offsets;

// stss atom
	unsigned long*      keyframe;
	unsigned long       num_keyframes;

// stts atom
	unsigned long       time_to_sample_count;
	QTTimeToSampleEntry*time_to_sample_table;
	unsigned long*      sample_map;

// stsz atom
	unsigned long*      sample_size;
	unsigned long       default_sample_size;
	unsigned long       num_samples;
	unsigned long       perceived_samples;
	QTSampleInfo*       sample_list;

// Sample properties
	unsigned long       dwFlags;
	unsigned long       dwMaxSample;
	__int64             i64TotalBytes;

// AVCDecoderConfigurationRecord or MPEG-4 Part 2 VOL prefix
	unsigned char*      avc_dcr;
	int                 dcr_size;
	int                 avc_len;

// Cached stream position
	__int64             i64bytepos;
	unsigned long       dwCurBlock;

	QTTrack();
	virtual ~QTTrack();

	static  bool    Verify(const QTTrack& qtt);
	virtual void    DeriveFrameRate();
	virtual void    DeriveSampleMap();
//	virtual void    DeriveTimeStamps();
	virtual bool    Finish();
};


/***************************************************************
 *          Handlers for supported audio types
 ***************************************************************/

class QTTrack_ac_3 : public QTTrack {
public:
	bool Finish();
};

class QTTrack_float : public QTTrack {
public:
	bool Finish();
};

class QTTrack__mp3 : public QTTrack {
public:
	bool Finish();
};

class QTTrack_ima4 : public QTTrack {
public:
	bool Finish();
};

class QTTrack_mp4a : public QTTrack {
public:
	bool Finish();
	void ParseHeader(unsigned int hdr);
};

class QTTrack_ms_u : public QTTrack {
public:
	bool Finish();
};

class QTTrack_ms11 : public QTTrack {
public:
	bool Finish();
};

class QTTrack_PCM : public QTTrack {
public:
	bool Finish();
};

class QTTrack_samr : public QTTrack {
public:
	bool Finish();
};

class QTTrack_ulaw : public QTTrack {
public:
	bool Finish();
};

#endif  // _QTTRACK_H_

