#ifndef _QTTYPES_H_
#define _QTTYPES_H_


// These swap the bytes in place. Note that the actual size
// of x is not checked, so you can (for example) swap just
// the least significant bytes of a larger variable.

#define bswap16(x) \
{	unsigned short ax = *((unsigned short *)&(x)); \
	*((unsigned short *)&(x)) = (ax >> 8) | (ax << 8); \
}

#define bswap32(x) \
{	unsigned long eax = *((unsigned long *)&(x)); \
	*((unsigned long *)&(x)) = (eax >> 24) | ((eax >> 8) & 0x0000FF00UL) | ((eax << 8) & 0x00FF0000UL) | (eax << 24); \
}

#define bswap64(x) \
{	unsigned long eax = *((unsigned long *)&(x) + 0); \
	unsigned long edx = *((unsigned long *)&(x) + 1); \
	*((unsigned long *)&(x) + 0) = (edx >> 24) | ((edx >> 8) & 0x0000FF00UL) | ((edx << 8) & 0x00FF0000UL) | (edx << 24); \
	*((unsigned long *)&(x) + 1) = (eax >> 24) | ((eax >> 8) & 0x0000FF00UL) | ((eax << 8) & 0x00FF0000UL) | (eax << 24); \
}

// Wave formats not defined in mmreg.h
#define WAVE_FORMAT_RAW_AAC1    0xFF
#define WAVE_FORMAT_SAMR        0x57
#define WAVE_FORMAT_SAWB        0x58

// Supported audio tags (bswapped, lowercase)

#define A__MP3 '3pm.'       // MPEG Layer III (CBR / VBR)
#define A_AC_3 '3-ca'       // Dolby Digital AC-3
#define A_ALAW 'wala'       // 8-bit A-law 2:1
#define A_FL32 '23lf'       // 32-bit floating point
#define A_FL64 '46lf'       // 64-bit floating point
#define A_IMA4 '4ami'       // QuickTime IMA ADPCM
#define A_IN24 '42ni'       // 24-bit PCM
#define A_IN32 '23ni'       // 32-bit PCM
#define A_MP4A 'a4pm'       // MP4 Audio
#define A_MS11 0x1100736D	// Microsoft IMA ADPCM
#define A_MS_U 'u\0sm'      // MPEG Layer III (CBR only)
#define A_RAW_ ' war'       // 8-bit raw PCM
#define A_SAMR 'rmas'       // Adaptive Multi-Rate (narrowband)
#define A_SAWB 'bwas'       // Adaptive Multi-Rate (wideband)
#define A_SOWT 'twos'       // 16-bit PCM (little-endian)
#define A_TWOS 'sowt'       // 16-bit PCM (big-endian)
#define A_ULAW 'walu'       // 8-bit mu-law 2:1

// Known but currently unsupported audio tags
/*
#define A_MAC3 '3cam'	// MACE 3.1 (Quinceanera.mov)
#define A_QDMC 'cmdq'	// QDesign Music v1
#define A_QDM2 '2mdq'	// QDesign Music v2
#define A_QCLP 'plcq'	// Qualcomm PureVoice (pd_3_25_02.mov)
*/

// Recognized video tags (bswapped, lowercase)

#define V_AVC1 '1cva'	// AVC
#define V_CVID 'divc'	// CinePak
#define V_DVC_ ' cvd'	// QuickTime NTSC DV-25
#define V_DVCP 'pcvd'	// QuickTime PAL DV-25
#define V_H263 '362h'	// H.263
#define V_IV41 '14vi'	// Indeo Video 4.1
#define V_JPEG 'gepj'	// JPEG
#define V_MJPA 'apjm'	// Motion JPEG (format A)
#define V_MJPB 'bpjm'	// Motion JPEG (format B)
#define V_MJPG 'gpjm'	// Motion JPEG
#define V_MP4V 'v4pm'	// MP4 Video
#define V_RLE_ ' elr'	// QuickTime RLE (ntsc_dv_smpte_bars.mov)
#define V_RPZA 'azpr'	// Apple Roadpizza
#define V_S263 '362s'	// 3GPP H.263
#define V_SVQ1 '1qvs'	// Sorenson v1
#define V_SVQ3 '3qvs'	// Sorenson v3
#define V_VC_1 '1-cv'	// Microsoft VC-1

// Known but currently unsupported video tags
/*
#define V_2VUY '2vuY'	// Uncompressed YCbCr 4:2:2 (?)
#define V_GIF_ ' fig'	// Graphics Interchange Format
#define V_KPCD 'dcpk'	// Kodak Photo CD
#define V_PNG_ ' gnp'	// Portable Network Graphics
#define V_RAW_ ' war'	// Uncompressed RGB
#define V_SMC_ ' cms'	// QuickTime Graphics
#define V_YUV2 '2vuy'	// Uncompressed YCbCr 4:2:2
#define V_TIFF 'ffit'	// Tagged Image File Format
#define V_V210 '012v'	// Uncompressed YCbCr 4:2:2 10-bit
#define V_V216 '612v'	// Uncompressed YCbCr 4:2:2 10, 12, 14, or 16-bit
#define V_V308 '803v'	// Uncompressed YCbCr 4:4:4 8-bit
#define V_V408 '804v'	// Uncompressed YCbCr 4:4:4:4 8-bit
#define V_V410 '014v'	// Uncompressed YCbCr 4:4:4 10-bit
*/


// Keyframe bit (in size field of QTSampleInfo)
#define QT_KEYFRAME 0x80000000UL


typedef struct {
	__int64         offset;
	__int64         size;
	unsigned long   type;
} QTAtom;


#include <PshPack1.h>

typedef struct {
	__int64         pos;
	unsigned long   size;
//	unsigned long   pts;
} QTSampleInfo;

typedef struct {
	unsigned long   first;
	unsigned long   samples;
	unsigned long   ID;
} QTChunk;

typedef struct {
	unsigned int    count;
	unsigned int    duration;
} QTTimeToSampleEntry;

typedef struct {
	DWORD   dwDataFormat;
	BYTE    bReserved[6];
	WORD    wDataReferenceIndex;
	WORD    wVersion;
	WORD    wRevisionLevel;
	DWORD   dwVendor;
	DWORD   dwTemporalQuality;
	DWORD   dwSpatialQuality;
	WORD    wWidth;
	WORD    wHeight;
	DWORD   dwHorizontalResolution;
	DWORD   dwVerticalResolution;
	DWORD   dwDataSize;
	WORD    wFrameCount;
	BYTE    bCompressorName[32];
	WORD    wDepth;
	WORD    wColorTableID;
} ImageDescription;

typedef struct {
	DWORD   dwDataFormat;
	BYTE    bReserved[6];
	WORD    wDataReferenceIndex;
	WORD    wVersion;
	WORD    wRevisionLevel;
	DWORD   dwVendor;
	WORD    wNumberOfChannels;
	WORD    wSampleSize;
	WORD    wCompressionID;
	WORD    wPacketSize;
	DWORD   dwSampleRate;
} SoundDescription;

#include <PopPack.h>


// stsd extensions
class SDExtensions {

public:
// video/audio codec extra data
	unsigned char*  pCodecPrivate;
	int             iCodecPrivateSize;

// avcC atom
	unsigned char*  avc_dcr;
	int             dcr_size;
	int             dcr_len;

// esds atom
	int             ObjectTypeID;
	int             DisplayWidth;
	int             DisplayHeight;

// pasp atom
	unsigned long   hSpacing;
	unsigned long   vSpacing;

// dac3 atom
	unsigned short  wAC3Channels;

// enda atom
	int             iEndianness;

	SDExtensions()
		: pCodecPrivate     (NULL)
		, iCodecPrivateSize (0)
		, avc_dcr           (NULL)
		, dcr_size          (0)
		, dcr_len           (0)
		, ObjectTypeID      (0)
		, DisplayWidth      (0)
		, DisplayHeight     (0)
		, hSpacing          (0)
		, vSpacing          (0)
		, wAC3Channels      (0)
		, iEndianness       (0)
	{}

	~SDExtensions()
	{
		delete[] avc_dcr;
	}
};


#endif  // _QTTYPES_H_

