#ifndef _VIDEOSOURCEQT_H_
#define _VIDEOSOURCEQT_H_


class VideoSourceQT : public vdxunknown<IVDXStreamSource>, public IVDXStreamSourceV3, public IVDXVideoSource {

private:
	InputFileQT    *const parentPtr;
	QTTrack        *const video;

	DWORD           dwPrevSample;
	unsigned char  *frame_buffer;

	BITMAPINFOHEADER bmDummy;

	IVDXStreamSource::ErrorMode errmode;

	bool IsDroppedFrame(unsigned long frame);
	int  ParseAVCFrame(const unsigned char *src, unsigned char *dst, int len);

public:
	VideoSourceQT(InputFileQT *const pp, QTTrack *const qtt);
	~VideoSourceQT();

	// IVDXUnknown methods
	int         VDXAPIENTRY AddRef();
	int         VDXAPIENTRY Release();
	void       *VDXAPIENTRY AsInterface(uint32 iid);

	// IVDXStreamSource methods
	void        VDXAPIENTRY GetStreamSourceInfo(VDXStreamSourceInfo&);
	bool        VDXAPIENTRY Read(sint64 lStart, uint32 lCount, void *lpBuffer, uint32 cbBuffer, uint32 *lBytesRead, uint32 *lSamplesRead);

	const void *VDXAPIENTRY GetDirectFormat();
	int         VDXAPIENTRY GetDirectFormatLen();

	ErrorMode   VDXAPIENTRY GetDecodeErrorMode();
	void        VDXAPIENTRY SetDecodeErrorMode(ErrorMode mode);
	bool        VDXAPIENTRY IsDecodeErrorModeSupported(ErrorMode mode);

	bool        VDXAPIENTRY IsVBR();
	sint64      VDXAPIENTRY TimeToPositionVBR(sint64 us);
	sint64      VDXAPIENTRY PositionToTimeVBR(sint64 samples);

	// IVDXStreamSourceV3 methods
	void        VDXAPIENTRY GetStreamSourceInfoV3(VDXStreamSourceInfoV3& info);

	// IVDXVideoSource methods
	void        VDXAPIENTRY GetVideoSourceInfo(VDXVideoSourceInfo& info);

	bool        VDXAPIENTRY CreateVideoDecoderModel(IVDXVideoDecoderModel **);
	bool        VDXAPIENTRY CreateVideoDecoder(IVDXVideoDecoder **);

	void        VDXAPIENTRY GetSampleInfo(sint64 sample_num, VDXVideoFrameInfo& frameInfo);
	bool        VDXAPIENTRY IsKey(sint64 sample_num);

	sint64      VDXAPIENTRY GetFrameNumberForSample(sint64 sample_num);
	sint64      VDXAPIENTRY GetSampleNumberForFrame(sint64 frame_num);
	sint64      VDXAPIENTRY GetRealFrame(sint64 frame_num);

	sint64      VDXAPIENTRY GetSampleBytePosition(sint64 sample_num);
};


class VideoSourceDummy : public vdxunknown<IVDXStreamSource>, public IVDXStreamSourceV3, public IVDXVideoSource {

private:
	InputFileQT    *const parentPtr;
	QTTrack        *const audio;

	BITMAPINFOHEADER bmDummy;

	IVDXStreamSource::ErrorMode errmode;

public:
	VideoSourceDummy(InputFileQT *const pp, QTTrack *const qtt);
	~VideoSourceDummy();

	// IVDXUnknown methods
	int         VDXAPIENTRY AddRef();
	int         VDXAPIENTRY Release();
	void       *VDXAPIENTRY AsInterface(uint32 iid);

	// IVDXStreamSource methods
	void        VDXAPIENTRY GetStreamSourceInfo(VDXStreamSourceInfo&);
	bool        VDXAPIENTRY Read(sint64 lStart, uint32 lCount, void *lpBuffer, uint32 cbBuffer, uint32 *lBytesRead, uint32 *lSamplesRead);

	const void *VDXAPIENTRY GetDirectFormat();
	int         VDXAPIENTRY GetDirectFormatLen();

	ErrorMode   VDXAPIENTRY GetDecodeErrorMode();
	void        VDXAPIENTRY SetDecodeErrorMode(ErrorMode mode);
	bool        VDXAPIENTRY IsDecodeErrorModeSupported(ErrorMode mode);

	bool        VDXAPIENTRY IsVBR();
	sint64      VDXAPIENTRY TimeToPositionVBR(sint64 us);
	sint64      VDXAPIENTRY PositionToTimeVBR(sint64 samples);

	// IVDXStreamSourceV3 methods
	void        VDXAPIENTRY GetStreamSourceInfoV3(VDXStreamSourceInfoV3& info);

	// IVDXVideoSource methods
	void        VDXAPIENTRY GetVideoSourceInfo(VDXVideoSourceInfo& info);

	bool        VDXAPIENTRY CreateVideoDecoderModel(IVDXVideoDecoderModel **);
	bool        VDXAPIENTRY CreateVideoDecoder(IVDXVideoDecoder **);

	void        VDXAPIENTRY GetSampleInfo(sint64 sample_num, VDXVideoFrameInfo& frameInfo);
	bool        VDXAPIENTRY IsKey(sint64 sample_num);

	sint64      VDXAPIENTRY GetFrameNumberForSample(sint64 sample_num);
	sint64      VDXAPIENTRY GetSampleNumberForFrame(sint64 frame_num);
	sint64      VDXAPIENTRY GetRealFrame(sint64 frame_num);

	sint64      VDXAPIENTRY GetSampleBytePosition(sint64 sample_num);
};


#endif  // _VIDEOSOURCEQT_H_

